/* General C interface to retrieve SLT information 

   Author: Brian Petersen
  
   Nov 13 1999 - Created 

   */
#ifndef SLTINFO_H
#define SLTINFO_H
#ifdef __cplusplus
extern "C" {
#endif

#define MAX_SLT_FEDS 1000

#include "etb.h"

#ifndef _L1DATA
typedef enum {         /* Copied from l1data */
  EPLUS,
  EMINUS,
  MUPLUS,
  MUMINUS,
  PIPLUS,
  PIMINUS
} PARTTYPE;
#endif

#define SLTMAX_TRKS_TRIG 2

typedef enum {      /* Return codes from SLT routines */
  SLT_OK,
  SLT_NODSLTBANK,
  SLT_DATAERROR,
  SLT_OUTOFRANGE
} SLTRC;

typedef enum {
 SLTEVENT_REJECT=0,
 SLTEVENT_ACCEPT,
 SLTEVENT_BYPASS,
 SLTEVENT_BYPASS_AND_ACCEPT,
} SLTSTATUS;

typedef enum {
  SLTTYPE_DIMUON,
  SLTTYPE_DIELECTRON
} SLTTRIGTYPE;

typedef enum {
  SLTTRIG_REJECT,
  SLTTRIG_ACCEPT
} SLTTRIGSTATUS;

typedef enum {
  SLTTRACK_ERROR,         /* Track not found or similar */
  SLTTRACK_ACCEPT,        /* This was a good track */
  SLTTRACK_REJECT_SILI,   /* Rejected by l2sili */
  SLTTRACK_REJECT_OTHER   /* Not accepted for other(unknown) reason */
} SLTTRACKSTATUS;


#define SLTTRIG_VERTEX 1          /* We have a good vertex (chisq) */
#define SLTTRIG_DETACHEDVERTEX 2  /* We have a detached vertex */


struct SLT_Status {
  SLTSTATUS status;
  int ntracks;      /* Number of tracks processed by the SLT*/
  int ntriggers;    /* Number of triggers (track pairs) processed by the SLT*/
};
typedef struct SLT_Status SLT_STATUS;

struct SLT_Trigger {
  SLTTRIGTYPE type;
  SLTTRIGSTATUS status;
  int info;          /* Vertex information (detached etc.) */
  int ntracks;
  int trackindex[SLTMAX_TRKS_TRIG]; /* Tracks numbers used by trigger */
};
typedef struct SLT_Trigger SLT_TRIGGER;

struct SLT_Track {
  SLTTRACKSTATUS status;       /* In principle we can add some general
				  information here like momentum and so on */
};
typedef struct SLT_Track SLT_TRACK;

struct SLT_SeedTrack {
  PARTTYPE trackID;      /* pretrigger track ID */
  float energy;          /* total energy reconstructed */
  float e_err;           /* error on energy, or 0. if unavailable */
  float pxpz;            /* Px/Pz */
  int charge;            /* charge */
  float x1;              /* The RoI definition: */
  float x1_err;
  float y1;
  float y1_err;
  float z1;
  float x2;
  float x2_err;
  float y2;
  float y2_err;
  float z2;
};
typedef struct SLT_SeedTrack SLT_SEEDTRACK;


struct SLT_RefitTrack {
  float tx;      /* Slope in x */
  float bx;      /* Intercept in x */
  float ty;      /* Slope in y */
  float by;      /* Intercept in y */
  float nhits;   /* Number of hits used */
  float chisq;   /* pseudo chi square */
};
typedef struct SLT_RefitTrack SLT_REFITTRACK;

struct SLT_SiliTrack {
  float pxpz;            
  float tx;      /* Slope in x */
  float bx;      /* Intercept in x */
  float ty;      /* Slope in y */
  float by;      /* Intercept in y */
  int nhitx;     /* Number of hits in x view */
  int nhity;     /* Number of hits in y view */
  float chix;    /* Chisq of x-view fit */
  float chiy;    /* Chisq of y-view fit */
};
typedef struct SLT_SiliTrack SLT_SILITRACK;

struct SLT_SiliTrackCov {
  float xcov[3];
  float ycov[3];
};
typedef struct SLT_SiliTrackCov SLT_SILITRACKCOV;


/* Function calls */

/* SLTRC SLTReadPipe(); */
SLTRC SLTReadDSLT();
SLTRC SLTReadSLT(EVENT_HDR *event);
SLTRC SLTReadEvent(ETB_fed *feds,int num);
SLTRC SLTStatus(SLT_STATUS *status);
SLTRC SLTTrigInfo(int trignum,SLT_TRIGGER *trig);
SLTRC SLTTrackInfo(int tracknum,SLT_TRACK *track);
SLTRC SLTSeedTrackInfo(int tracknum,SLT_SEEDTRACK *track);
SLTRC SLTRefitTrackInfo(int tracknum,SLT_REFITTRACK *track);
SLTRC SLTSiliTrackInfo(int tracknum,SLT_SILITRACK *track);
SLTRC SLTSiliTrackCovInfo(int tracknum,SLT_SILITRACKCOV *track);
int SLT_TriggerCount(SLTTRIGTYPE type);
SLTRC SLTPipeFeds(int max, ETB_fed *feds,int *n, int *bc);
SLTRC SLTPipeToDSLT();

#ifdef __cplusplus
}
#endif

#endif
